@@ -77,6 +77,7 @@ class Book(models.Model):
 class Promo(models.Model):
     name = models.CharField(max_length=100, verbose_name='¿Name?')
     book = models.ForeignKey(Book, models.CASCADE)
+    author = models.ForeignKey(User, models.SET_NULL, blank=True, null=True)
 
     def __str__(self):
         return self.name
@@ -100,6 +101,7 @@ class Chapter(models.Model):
 class ChapterXtra1(models.Model):
     chap = models.OneToOneField(Chapter, models.CASCADE, verbose_name='¿Chap?')
     xtra = models.CharField(max_length=100, verbose_name='¿Xtra?')
+    guest_author = models.ForeignKey(User, models.SET_NULL, blank=True, null=True)
 
     def __str__(self):
         return '¿Xtra1: %s' % self.xtra
@@ -397,7 +399,7 @@ class Recommender(Title):
 
 
 class Recommendation(Title):
-    recommender = models.ForeignKey(Recommender, models.CASCADE)
+    the_recommender = models.ForeignKey(Recommender, models.CASCADE)
 
 
 class Collector(models.Model):
@@ -596,9 +598,13 @@ class CyclicTwo(models.Model):
         return self.name
 
 
+@python_2_unicode_compatible
 class Topping(models.Model):
     name = models.CharField(max_length=20)
 
+    def __str__(self):
+        return self.name
+
 
 class Pizza(models.Model):
     name = models.CharField(max_length=20)
@@ -621,6 +627,7 @@ class WorkHour(models.Model):
 
 class Question(models.Model):
     question = models.CharField(max_length=20)
+    posted = models.DateField(default=datetime.date.today)
 
 
 @python_2_unicode_compatible
@@ -908,11 +915,16 @@ class ChildOfReferer(ParentWithFK):
 
 
 # Models for #23431
+class InlineReferer(models.Model):
+    pass
+
+
 class ReferencedByInline(models.Model):
     name = models.CharField(max_length=20, unique=True)
 
 
 class InlineReference(models.Model):
+    referer = models.ForeignKey(InlineReferer, models.CASCADE)
     fk = models.ForeignKey(
         ReferencedByInline,
         models.CASCADE,
@@ -921,10 +933,6 @@ class InlineReference(models.Model):
     )
 
 
-class InlineReferer(models.Model):
-    refs = models.ManyToManyField(InlineReference)
-
-
 # Models for #23604 and #23915
 class Recipe(models.Model):
     rname = models.CharField(max_length=20, unique=True)
