@@ -1,9 +1,10 @@
 """
 Tests for F() query expression syntax.
 """
-
 from __future__ import unicode_literals
 
+import uuid
+
 from django.db import models
 from django.utils.encoding import python_2_unicode_compatible
 
@@ -61,6 +62,15 @@ class Experiment(models.Model):
         return self.end - self.start
 
 
+@python_2_unicode_compatible
+class Result(models.Model):
+    experiment = models.ForeignKey(Experiment, models.CASCADE)
+    result_time = models.DateTimeField()
+
+    def __str__(self):
+        return "Result at %s" % self.result_time
+
+
 @python_2_unicode_compatible
 class Time(models.Model):
     time = models.TimeField(null=True)
@@ -69,9 +79,24 @@ class Time(models.Model):
         return "%s" % self.time
 
 
+@python_2_unicode_compatible
+class SimulationRun(models.Model):
+    start = models.ForeignKey(Time, models.CASCADE, null=True, related_name='+')
+    end = models.ForeignKey(Time, models.CASCADE, null=True, related_name='+')
+    midpoint = models.TimeField()
+
+    def __str__(self):
+        return "%s (%s to %s)" % (self.midpoint, self.start, self.end)
+
+
+class UUIDPK(models.Model):
+    id = models.UUIDField(primary_key=True, default=uuid.uuid4)
+
+
 @python_2_unicode_compatible
 class UUID(models.Model):
     uuid = models.UUIDField(null=True)
+    uuid_fk = models.ForeignKey(UUIDPK, models.CASCADE, null=True)
 
     def __str__(self):
         return "%s" % self.uuid
