@@ -78,7 +78,7 @@ class Annotation(models.Model):
 @python_2_unicode_compatible
 class ExtraInfo(models.Model):
     info = models.CharField(max_length=100)
-    note = models.ForeignKey(Note, models.CASCADE)
+    note = models.ForeignKey(Note, models.CASCADE, null=True)
     value = models.IntegerField(null=True)
 
     class Meta:
@@ -126,6 +126,10 @@ class Report(models.Model):
         return self.name
 
 
+class ReportComment(models.Model):
+    report = models.ForeignKey(Report, models.CASCADE)
+
+
 @python_2_unicode_compatible
 class Ranking(models.Model):
     rank = models.IntegerField()
@@ -526,8 +530,8 @@ class Job(models.Model):
 
 
 class JobResponsibilities(models.Model):
-    job = models.ForeignKey(Job, models.SET_NULL, to_field='name')
-    responsibility = models.ForeignKey('Responsibility', models.SET_NULL, to_field='description')
+    job = models.ForeignKey(Job, models.CASCADE, to_field='name')
+    responsibility = models.ForeignKey('Responsibility', models.CASCADE, to_field='description')
 
 
 @python_2_unicode_compatible
@@ -620,7 +624,7 @@ class Order(models.Model):
 
 @python_2_unicode_compatible
 class OrderItem(models.Model):
-    order = models.ForeignKey(Order, models.SET_NULL, related_name='items')
+    order = models.ForeignKey(Order, models.CASCADE, related_name='items')
     status = models.IntegerField()
 
     class Meta:
@@ -637,8 +641,8 @@ class BaseUser(models.Model):
 @python_2_unicode_compatible
 class Task(models.Model):
     title = models.CharField(max_length=10)
-    owner = models.ForeignKey(BaseUser, models.SET_NULL, related_name='owner')
-    creator = models.ForeignKey(BaseUser, models.SET_NULL, related_name='creator')
+    owner = models.ForeignKey(BaseUser, models.CASCADE, related_name='owner')
+    creator = models.ForeignKey(BaseUser, models.CASCADE, related_name='creator')
 
     def __str__(self):
         return self.title
@@ -654,7 +658,7 @@ class Staff(models.Model):
 
 @python_2_unicode_compatible
 class StaffUser(BaseUser):
-    staff = models.OneToOneField(Staff, models.SET_NULL, related_name='user')
+    staff = models.OneToOneField(Staff, models.CASCADE, related_name='user')
 
     def __str__(self):
         return self.staff
