@@ -1,16 +1,22 @@
+from __future__ import unicode_literals
+
+import gettext
 import os
 import shutil
+import sys
 import tempfile
 from importlib import import_module
 
 from django import conf
 from django.contrib import admin
-from django.test import SimpleTestCase, override_settings
+from django.test import SimpleTestCase, mock, override_settings
 from django.test.utils import extend_sys_path
 from django.utils import autoreload
-from django.utils._os import npath
+from django.utils._os import npath, upath
+from django.utils.six.moves import _thread
+from django.utils.translation import trans_real
 
-LOCALE_PATH = os.path.join(os.path.dirname(__file__), 'locale')
+LOCALE_PATH = os.path.join(os.path.dirname(upath(__file__)), 'locale')
 
 
 class TestFilenameGenerator(SimpleTestCase):
@@ -42,16 +48,16 @@ class TestFilenameGenerator(SimpleTestCase):
 
     def test_django_locales(self):
         """
-        Test that gen_filenames() yields the built-in Django locale files.
+        gen_filenames() yields the built-in Django locale files.
         """
-        django_dir = os.path.join(os.path.dirname(conf.__file__), 'locale')
+        django_dir = os.path.join(os.path.dirname(upath(conf.__file__)), 'locale')
         django_mo = os.path.join(django_dir, 'nl', 'LC_MESSAGES', 'django.mo')
         self.assertFileFound(django_mo)
 
     @override_settings(LOCALE_PATHS=[LOCALE_PATH])
     def test_locale_paths_setting(self):
         """
-        Test that gen_filenames also yields from LOCALE_PATHS locales.
+        gen_filenames also yields from LOCALE_PATHS locales.
         """
         locale_paths_mo = os.path.join(LOCALE_PATH, 'nl', 'LC_MESSAGES', 'django.mo')
         self.assertFileFound(locale_paths_mo)
@@ -59,12 +65,11 @@ class TestFilenameGenerator(SimpleTestCase):
     @override_settings(INSTALLED_APPS=[])
     def test_project_root_locale(self):
         """
-        Test that gen_filenames also yields from the current directory (project
-        root).
+        gen_filenames() also yields from the current directory (project root).
         """
         old_cwd = os.getcwd()
         os.chdir(os.path.dirname(__file__))
-        current_dir = os.path.join(os.path.dirname(__file__), 'locale')
+        current_dir = os.path.join(os.path.dirname(upath(__file__)), 'locale')
         current_dir_mo = os.path.join(current_dir, 'nl', 'LC_MESSAGES', 'django.mo')
         try:
             self.assertFileFound(current_dir_mo)
@@ -74,9 +79,9 @@ class TestFilenameGenerator(SimpleTestCase):
     @override_settings(INSTALLED_APPS=['django.contrib.admin'])
     def test_app_locales(self):
         """
-        Test that gen_filenames also yields from locale dirs in installed apps.
+        gen_filenames() also yields from locale dirs in installed apps.
         """
-        admin_dir = os.path.join(os.path.dirname(admin.__file__), 'locale')
+        admin_dir = os.path.join(os.path.dirname(upath(admin.__file__)), 'locale')
         admin_mo = os.path.join(admin_dir, 'nl', 'LC_MESSAGES', 'django.mo')
         self.assertFileFound(admin_mo)
 
@@ -86,7 +91,7 @@ class TestFilenameGenerator(SimpleTestCase):
         If i18n machinery is disabled, there is no need for watching the
         locale files.
         """
-        django_dir = os.path.join(os.path.dirname(conf.__file__), 'locale')
+        django_dir = os.path.join(os.path.dirname(upath(conf.__file__)), 'locale')
         django_mo = os.path.join(django_dir, 'nl', 'LC_MESSAGES', 'django.mo')
         self.assertFileNotFound(django_mo)
 
@@ -185,3 +190,81 @@ class TestFilenameGenerator(SimpleTestCase):
             with self.assertRaises(Exception):
                 autoreload.check_errors(import_module)('test_exception')
         self.assertFileFound(filename)
+
+
+class CleanFilesTests(SimpleTestCase):
+    TEST_MAP = {
+        # description: (input_file_list, expected_returned_file_list)
+        'falsies': ([None, False], []),
+        'pycs': (['myfile.pyc'], ['myfile.py']),
+        'pyos': (['myfile.pyo'], ['myfile.py']),
+        '$py.class': (['myclass$py.class'], ['myclass.py']),
+        'combined': (
+            [None, 'file1.pyo', 'file2.pyc', 'myclass$py.class'],
+            ['file1.py', 'file2.py', 'myclass.py'],
+        )
+    }
+
+    def _run_tests(self, mock_files_exist=True):
+        with mock.patch('django.utils.autoreload.os.path.exists', return_value=mock_files_exist):
+            for description, values in self.TEST_MAP.items():
+                filenames, expected_returned_filenames = values
+                self.assertEqual(
+                    autoreload.clean_files(filenames),
+                    expected_returned_filenames if mock_files_exist else [],
+                    msg='{} failed for input file list: {}; returned file list: {}'.format(
+                        description, filenames, expected_returned_filenames
+                    ),
+                )
+
+    def test_files_exist(self):
+        """
+        If the file exists, any compiled files (pyc, pyo, $py.class) are
+        transformed as their source files.
+        """
+        self._run_tests()
+
+    def test_files_do_not_exist(self):
+        """
+        If the files don't exist, they aren't in the returned file list.
+        """
+        self._run_tests(mock_files_exist=False)
+
+
+class ResetTranslationsTests(SimpleTestCase):
+
+    def setUp(self):
+        self.gettext_translations = gettext._translations.copy()
+        self.trans_real_translations = trans_real._translations.copy()
+
+    def tearDown(self):
+        gettext._translations = self.gettext_translations
+        trans_real._translations = self.trans_real_translations
+
+    def test_resets_gettext(self):
+        gettext._translations = {'foo': 'bar'}
+        autoreload.reset_translations()
+        self.assertEqual(gettext._translations, {})
+
+    def test_resets_trans_real(self):
+        trans_real._translations = {'foo': 'bar'}
+        trans_real._default = 1
+        trans_real._active = False
+        autoreload.reset_translations()
+        self.assertEqual(trans_real._translations, {})
+        self.assertIsNone(trans_real._default)
+        self.assertIsInstance(trans_real._active, _thread._local)
+
+
+class TestRestartWithReloader(SimpleTestCase):
+
+    def test_environment(self):
+        """"
+        With Python 2 on Windows, restart_with_reloader() coerces environment
+        variables to str to avoid "TypeError: environment can only contain
+        strings" in Python's subprocess.py.
+        """
+        # With unicode_literals, these values are unicode.
+        os.environ['SPAM'] = 'spam'
+        with mock.patch.object(sys, 'argv', ['-c', 'pass']):
+            autoreload.restart_with_reloader()
